<?php

namespace ACL\Repositories;


use ACL\Traits\TQuery;
use ACL\Helpers\Instances\InstanceProvider;
use Doctrine\ORM\EntityManager;
use Exception;

class Perfilacesso extends RepositoryBase {

    Use TQuery;

    protected $table = 'PERFILACESSO';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    public function getNrorg() {
        return InstanceProvider::getEnvironment()->getCurrentOrganizationId();
    }

    /**
     * @param $organizationId
     * @return array
     */
    public function findAllPerfilacesso($organizationId): array
    {
        $query = "SELECT NRPERFILACESSO, NMPERFILACESSO, IDPERFILACESSOADM, NRORG, IDATIVO
                    FROM PERFILACESSO
                   WHERE NRORG = :NRORG";

        $params = array('NRORG' => $organizationId);

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function createPerfilacesso($newNrperfilacesso, $nmperfilacesso, $idperfilacessoadm, $idativo)
    {
        $arrayData = array(
            'NRPERFILACESSO' => intval($newNrperfilacesso),
            'NMPERFILACESSO' => $nmperfilacesso,
            'IDPERFILACESSOADM' => $idperfilacessoadm,
            'IDATIVO' => $idativo
        );

        $arrayData = $this->auditValuesFieldArrayMerge($arrayData, $this->getAuditFieldValuesDefaultInsert());

        $sql = $this->buildInsert($arrayData);

        $this->insert($sql, $arrayData);
    }

    /**
     * @param $nrperfilacesso
     * @param $nmperfilacesso
     * @param $idperfilacessoadm
     * @param $idativo
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
    public function updatePerfilacesso($nrperfilacesso, $nmperfilacesso, $idperfilacessoadm, $idativo)
    {
        $arrayData = array(
            'NMPERFILACESSO' => $nmperfilacesso,
            'IDPERFILACESSOADM' => $idperfilacessoadm,
            'IDATIVO' => $idativo
        );

        $arrayData = $this->auditValuesFieldArrayMerge($arrayData, $this->getAuditFieldValuesDefaultUpdate());

        $arrayConditions = array(
            'NRPERFILACESSO' => intval($nrperfilacesso),
            'NRORG' => intval(InstanceProvider::getLibEnvironment()->getCurrentOrganizationId()),
        );

        $sql = $this->buildUpdate($arrayData, $arrayConditions);
        $arrayData['NRPERFILACESSO'] = intval($nrperfilacesso);
        $arrayData['NRORG'] = intval(InstanceProvider::getLibEnvironment()->getCurrentOrganizationId());

        $this->update($sql, $arrayData);
    }

    public function deletePerfilacesso($nrperfilacesso)
    {
        $arrayConditions = array(
            'NRPERFILACESSO' => intval($nrperfilacesso),
            'NRORG' => intval(InstanceProvider::getLibEnvironment()->getCurrentOrganizationId()),
        );

        $sql = $this->buildDelete($arrayConditions);
        $this->delete($sql, $arrayConditions);
    }

    /**
     * @param $nrorg
     * @return false|array
     * @throws Exception
     */
    public function findAccessControlUtilization($nrorg)
    {
        try {
            $query = "SELECT IDUTCTRLACESWEB
                        FROM PARAMGERAL
                       WHERE NRORG = $nrorg
                       AND IDUTCTRLACESWEB = 'S'";
            return $this->entityManager->getConnection()->fetchAllAssociative($query);
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }
    }

    /**
     * @param $nrorg
     * @param $idativo
     * @param $nrparcnegocio
     * @return array
     * @throws Exception
     */
    public function findPerfilacessoByOperador($nrorg, $idoperativoFilter, $nrparcnegocio): array
    {
        $query = "SELECT CDOPERULTATU, NRORGULTATU, CDOPERINCLUSAO, NRORGINCLUSAO, DTULTATU, DTINCLUSAO, IDATIVO, NRORG,
                         DTVALPERFILPARC, NRPERFILACESSO, NRPARCNEGOCIO, NRPERFACESSPARC, NMPERFILACESSO, TO_CHAR(DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE
                    FROM (SELECT PEPA.CDOPERULTATU,  PEPA.NRORGULTATU,  PEPA.CDOPERINCLUSAO,  PEPA.NRORGINCLUSAO,  PEPA.DTULTATU,
                                 PEPA.DTINCLUSAO,  PERF.IDATIVO, PEPA.NRORG,  PEPA.DTVALPERFILPARC,  PEPA.NRPERFILACESSO,
                                 PEPA.NRPARCNEGOCIO,  PEPA.NRPERFACESSPARC,  PERF.NMPERFILACESSO, PEPA.DTVALIDADE
                            FROM PERFACESSPARC PEPA
                            JOIN PERFILACESSO PERF ON PERF.NRPERFILACESSO = PEPA.NRPERFILACESSO AND PERF.NRORG = PEPA.NRORG)
                   WHERE NRORG = :NRORG
                     AND NRPARCNEGOCIO = :NRPARCNEGOCIO
                     $idoperativoFilter";

        $params = array(
            'NRORG'   => $nrorg,
            'NRPARCNEGOCIO' => $nrparcnegocio,
        );

        try {
            return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    /**
     * @param $nrorg
     * @param $nrparcnegocio
     * @return array|mixed[]
     * @throws Exception
     */
    public function findPerfilAcessoForAddOperador($nrorg, $nrparcnegocio): array
    {
        $query = "SELECT NRPERFILACESSO, NMPERFILACESSO, IDPERFILACESSOADM 
                    FROM PERFILACESSO 
                   WHERE NRORG = :NRORG  
                     AND IDATIVO != 'N'
                     AND NRPERFILACESSO NOT IN (SELECT NRPERFILACESSO FROM PERFACESSPARC WHERE :NRPARCNEGOCIO = NRPARCNEGOCIO)
                ORDER BY NMPERFILACESSO ASC";

        $params = array(
            "NRORG"         => $nrorg,
            "NRPARCNEGOCIO" => $nrparcnegocio
        );

        try {
            return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function findValAntigoPerfAcess($partnerAccessProfileNumber)
    {

        $query = "SELECT *FROM PERFILACESSO
                  WHERE NRPERFILACESSO = :NRPERFILACESSO
                  AND NRORG = :NRORG";

        $params = array (
            "NRPERFILACESSO" => $partnerAccessProfileNumber,
            "NRORG"         => $this->getNrorg(),
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    }

}
