<?php

namespace ACL\Services;

use ACL\Helpers\Instances\InstanceProvider;
use ACL\Repositories\ConfigBot as ConfigBotRepository;
use Exception;
use ACL\Util\AclUtilities;

class ConfigBot {

    /**
     * @var \Teknisa\Libs\Util\Environment
     */
    private $libEnvironment;

    /**
     * @var ConfigBotRepository
     */
    private $configBotRepository;

    public function __construct(ConfigBotRepository $configBotRepository)
    {
        $this->configBotRepository = $configBotRepository;
        $this->libEnvironment =  InstanceProvider::getLibEnvironment();
    }

    /**
     * @return array
     */
    public function findAllConfigBot($dynamicFilter, $page, $limit, $search,  $searchIn): array
    {
        $conditionsQuery = '';
        $groupIdString = '';
        $conditionsSearchQuery = '';
        if (!empty($dynamicFilter)) {
            $dynamicFilter = json_decode(utf8_encode(base64_decode($dynamicFilter)),true);
        }
        if (!empty($dynamicFilter["GROUP_ID"])) {
            $filterGroup = array (
                "TELEGRAM_GROUP_ID" => $dynamicFilter["GROUP_ID"],
            );
            $filterTempGroup = $this->configBotRepository->createDynamicFilter($filterGroup, null, false);
            $groupIdString = "AND TB.ID IN (SELECT TELEGRAM_BOT_ID FROM TELEGRAM_BOT_GROUP $filterTempGroup)";
            unset($dynamicFilter["GROUP_ID"]);
        }
        if(!empty($search) && !empty($searchIn)){
            $conditionsSearchQuery = $this->configBotRepository->createSearch($search, $searchIn);
        }
        if (!empty($dynamicFilter)) {
            $conditionsQuery = $this->configBotRepository->createDynamicFilter($dynamicFilter, $conditionsSearchQuery, false);
        }
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        return $this->configBotRepository->findAllConfigBot($conditionsQuery, $groupIdString, $page, $limit, $conditionsSearchQuery);
    }

    /**
     * @return array
     */
    public function findAllGroupBot(): array
    {
        return $this->configBotRepository->findAllGroupBot();
    }

    public function saveLogLibBOT($token, $id, $botId, $name, $username, $labels) {
        $valores = [$labels['label'], $token, $botId, $name, $username];
        $nomes = ["Tela", $labels['TOKEN'], $labels['BOT_ID'], $labels['NAME'], $labels['USERNAME']];
        $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($id, $descricao, '1', 'TELEGRAM_BOT', '/saveConfigBot');
    }

    public function saveLogLibGROUP($id, $idGroup, $name, $labels) {
        $valores = [$labels['label'], $idGroup, $name];
        $nomes = ["Tela", $labels['ID_GROUP'], $labels['NAME']];
        $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($id, $descricao, '1', 'TELEGRAM_GROUP', '/saveConfigBot');
    }

    public function saveLogLibBOTGROUP($idBotGroup, $idBot, $idGroup, $labels) {
        $valores = [$labels['label'], $idBot, $idGroup];
        $nomes = ["Tela", 'TELEGRAM_BOT_ID', 'TELEGRAM_GROUP_ID'];
        $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($idBotGroup, $descricao, '1', 'TELEGRAM_BOT_GROUP', '/saveConfigBot');
    }

    public function updateLogLib($description, $id, $idativo, $labels) {
        $botAntigo =  $this->configBotRepository->findBotAntigo($id);
        $arrayValores = [$labels['label']];
        $arrayNomes = ['Tela'];
        if ($botAntigo[0]['DESCRIPTION'] !== $description) {
            array_push($arrayValores, $description);
            array_push($arrayNomes, $labels['DESCRIPTION']);
        }
        if ($botAntigo[0]['IDATIVO'] !== $idativo) {
            array_push($arrayValores, $idativo);
            array_push($arrayNomes, $labels['IDATIVO']);
        }
        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister($id, $descricao, '2', 'TELEGRAM_BOT', '/saveConfigBot');
    }

    public function configBotSave($token, $id, $botId, $idativo, $name, $username,  $description, $groups, $groupsId, $labels)
    {   
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {
            if ($id == null) {
                $id = InstanceProvider::getNewCode()->getCodeByTableName('TELEGRAM_BOT', getNrorg(), 1, 10, true);
                $this->configBotRepository->configBotInsert($token, $id, $botId, $idativo, $name, $username, 
                $description, getNrorg(), getCurrentUserId());
                $this->saveLogLibBOT($token, $id, $botId, $name, $username, $labels);
                if (!empty($groups)) {
                    foreach ($groups as $group) {
                        $idGroup = InstanceProvider::getNewCode()->getCodeByTableName('TELEGRAM_GROUP', getNrorg(), 1, 10, true);
                        $this->configBotRepository->configGroupInsert($idGroup, $group["ID_GROUP"], $group["NAME"], getNrorg(), getCurrentUserId(), $idativo);
                        $this->saveLogLibGROUP($idGroup, $group["ID_GROUP"], $group["NAME"], $labels);
                        $idBotGroup = InstanceProvider::getNewCode()->getCodeByTableName('TELEGRAM_BOT_GROUP', getNrorg(), 1, 10, true);
                        $this->configBotRepository->configBotGroupInsert($idBotGroup, $id, $idGroup, getNrorg(), getCurrentUserId(), $idativo);
                        $this->saveLogLibBOTGROUP($idBotGroup, $id, $idGroup, $labels);
                    }
                }
            }else{
                if (!empty($groups) || true) {
                    $botGroup = array("TELEGRAM_BOT_ID" => $id);
                    $this->configBotRepository->deleteBotGroup($botGroup);
                    foreach ($groups as $group) {
                        foreach ($groupsId as $groupId) {
                            $groupData = array( "ID" => $groupId);
                            $this->configBotRepository->deleteGroup($groupData);
                        }
                        $idGroup = InstanceProvider::getNewCode()->getCodeByTableName('TELEGRAM_GROUP', getNrorg(), 1, 10, true);
                        $this->configBotRepository->configGroupInsert($idGroup, $group["ID_GROUP"], $group["NAME"], getNrorg(), getCurrentUserId(), $idativo);
                        $this->saveLogLibGROUP($idGroup, $group["ID_GROUP"], $group["NAME"], $labels);
                        $idBotGroup = InstanceProvider::getNewCode()->getCodeByTableName('TELEGRAM_BOT_GROUP', getNrorg(), 1, 10, true);
                        $this->configBotRepository->configBotGroupInsert($idBotGroup, $id, $idGroup, getNrorg(), getCurrentUserId(), $idativo);
                        $this->saveLogLibBOTGROUP($idBotGroup, $id, $idGroup, $labels);
                    }
                }
                $this->updateLogLib($description, $id, $idativo, $labels);
                $this->configBotRepository->configBotUpdate($description, $id, $idativo, getCurrentUserId());
            }
            $entityManager->flush();
            $connection->commit();
        }catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function deleteLogLibBotGroup($idBotGroup, $labels, $name) {
        $valores = [$labels['label'], $idBotGroup, $name];
        $nomes = ["Tela", 'ID', $labels['NAME']];
        $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($idBotGroup, $descricao, '3', 'TELEGRAM_BOT_GROUP', '/deleteConfigBot');
    }

    public function deleteLogLibGroup($idGroup, $labels,  $name) {
        $valores = [$labels['label'], $idGroup, $name];
        $nomes = ["Tela", 'ID', $labels['NAME']];
        $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($idGroup, $descricao, '3', 'TELEGRAM_GROUP', '/deleteConfigBot');
    }

    public function deleteLogLibBot($idBot, $labels,  $name) {
        $valores = [$labels['label'], $idBot, $name];
        $nomes = ["Tela", 'ID', $labels['NAME']];
        $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($idBot, $descricao, '3', 'TELEGRAM_BOT', '/deleteConfigBot');
    }

    public function configBotDelete($params, $labels)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {            
            foreach ($params as $row) {
                $idBotGroup = array( "TELEGRAM_BOT_ID" => $row["ID"]);
                $this->configBotRepository->deleteBotGroup($idBotGroup);
                $this->deleteLogLibBotGroup($row["ID"], $labels, $row["NAME"]);
                foreach ($row["GROUP_ID"] as $group) {
                    $idGroup = array( "ID" => $group);
                    $this->configBotRepository->deleteGroup($idGroup);
                    $this->deleteLogLibGroup($group, $labels, $row["NAME"]);
                }
                $idBot = array( "ID" => $row["ID"]);
                $this->configBotRepository->deleteBot($idBot);
                $this->deleteLogLibBot($row["ID"], $labels, $row["NAME"]);
            }
            $connection->commit();
        }catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function getDateGroupBot($token)
    {
        $bot = new \Zeedhi\MessageApi\Telegram\Manager($token);
        $allChats = $bot->getAllChatsByUpdates();
        $dataGroups = [];
        $idgroups = [];
        foreach ($allChats as $chat) {
            if (($chat->getType() == 'supergroup' || $chat->getType() == 'group')) {
                $idgroups[] = $chat->getId();
                $dataGroups[] = [
                    "ID_GROUP"  => $chat->getId(),
                    "NAME"      => $chat->getTitle() 
                ];
            }
        }
        $result = $dataGroups;
        $resultbanco = $this->configBotRepository->findGroupBotByToken($token,$idgroups);
        $result = array_merge($result,$resultbanco);
        $total = count($result);
        $returnNext = [ 'data' => $result, 'pagination' => [ 'limit' => $total, 'page' => 1, 'total' => $total]];
        return $returnNext;
    }
}