<?php

namespace ACL\Services;

use ACL\Helpers\Instances\InstanceProvider;
use ACL\Repositories\Perfacessparc as PerfacessparcRepository;
use ACL\Util\AclUtilities;
use Exception;

class Perfacessparc {

    /**
     * @var \Teknisa\Libs\Util\Environment
     */
    private $libEnvironment;

    /**
     * @var PerfacessparcRepository
     */
    private $perfacessparcRepository;

    public function __construct(PerfacessparcRepository $perfacessparcRepository)
    {
        $this->perfacessparcRepository = $perfacessparcRepository;
        $this->libEnvironment =  InstanceProvider::getLibEnvironment();
    }

    /**
     * @param $rows
     * @return array
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
    public function savePerfaccessparc($rows, $labels): array
    {
        $organizationId = $this->libEnvironment->getCurrentOrganizationId();

        foreach ($rows as $row) {
            $this->setParcnegocio($row);

            $row['NRPERFACESSPARC'] = InstanceProvider::getNewCode()->getCodeByTableName('PERFACESSPARC', $organizationId);
            $row = $this->perfacessparcRepository->auditValuesFieldArrayMerge($row,
                                                                        $this->perfacessparcRepository->getAuditFieldValuesDefaultInsert());
            $this->perfacessparcRepository->savePerfaccessparc($row['NRPERFACESSPARC'], $row['NRORG'],
                                                                $row['NRPARCNEGOCIO'], $row['NRPERFILACESSO'],
                                                                $row['CDOPERINCLUSAO'], $row['DTVALIDADE']);
            $operador = $this->perfacessparcRepository->findOperadorPerfilAcess($row['NRPARCNEGOCIO']);
            if (isset($row['NMPERFILACESSO'])) {
                $nmPerfilAcesso = $row['NMPERFILACESSO'];
            } else {
                $nmPerfilAcessoAux = $this->perfacessparcRepository->findPerfilAcess($row['NRPERFILACESSO']);
                $nmPerfilAcesso = $nmPerfilAcessoAux[0]['NMPERFILACESSO'];
                $labels['label'] = 'Perfil de Acesso';
            }
            $arrayValores = [$labels['label'], $operador[0]['CDOPERADOR'], $operador[0]['NMOPERADOR'], $nmPerfilAcesso, $row['NRPERFACESSPARC'], $row['NRPARCNEGOCIO'], $row['NRPERFILACESSO'], $row['DTVALIDADE']];
            $arrayNomes = ['Tela', 'Cod.Operador', 'Operador', 'Perfil',$labels['NRPERFACESSPARC'], $labels['NRPARCNEGOCIO'], $labels['NRPERFILACESSO'], $labels['DTVALIDADE']];
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($row['NRPERFACESSPARC'], $descricao, '1', 'PERFACESSPARC', '/perfacessparc');
                                                
        }
        return [];
    }

    /**
     * @param $params
     * @return array
     * @throws Exception|\Doctrine\DBAL\Driver\Exception
     */
    public function updatePerfaccessparc($params, $labels): array
    {
        foreach ($params as $row) {
            $perfAcessoOld = $this->perfacessparcRepository->findValAntigoPerfAcess($row['NRPERFACESSPARC']);
            if ($perfAcessoOld[0]['DTVALIDADE'] !== $row['DTVALIDADE']) {
                $operador = $this->perfacessparcRepository->findOperadorPerfilAcess($row['NRPARCNEGOCIO']);
                $arrayValores = [$labels['label'], $operador[0]['CDOPERADOR'], $operador[0]['NMOPERADOR'], $row['NMPERFILACESSO'], $row['NRPARCNEGOCIO'], $row['DTVALIDADE']];
                $arrayNomes = ['Tela', 'Cod.Operador', 'Operador', 'Perfil', $labels['NRPARCNEGOCIO'], $labels['DTVALIDADE']];
                $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
                InstanceProvider::getLibLog()->logRegister($row['NRPERFACESSPARC'], $descricao, '2', 'PERFACESSPARC', '/perfacessparc');
            }
            $this->perfacessparcRepository->updatePerfaccessparc(
                $row['NRPERFACESSPARC'], $row['DTVALIDADE'], getNrorg()
            );
        }
        return [];
    }

    public function deletePerfaccessparc($rows, $labels)
    {
        foreach ($rows as $row) {
            $operador = $this->perfacessparcRepository->findOperadorPerfilAcess($row['NRPARCNEGOCIO']);
            if (isset($row['NMPERFILACESSO'])) {
                $nmPerfilAcesso = $row['NMPERFILACESSO'];
            } else {
                $nmPerfilAcessoAux = $this->perfacessparcRepository->findPerfilAcess($row['NRPERFILACESSO']);
                $nmPerfilAcesso = $nmPerfilAcessoAux[0]['NMPERFILACESSO'];
                $labels['label'] = 'Perfil de Acesso';
            }
            $arrayValores = [$labels['label'], $operador[0]['CDOPERADOR'], $operador[0]['NMOPERADOR'], $nmPerfilAcesso, $row['NRPARCNEGOCIO'], $row['NRPERFACESSPARC']];
            $arrayNomes = ['Tela',  'Cod.Operador', 'Operador', 'Perfil', $labels['NRPARCNEGOCIO'], $labels['NRPERFACESSPARC']];
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($row['NRPERFACESSPARC'], $descricao, '3', 'PERFACESSPARC', '/perfacessparc');
            $this->perfacessparcRepository->deletePerfaccessparc($row['NRPERFACESSPARC'], getNrorg());
        }
    }

    private function setParcnegocio(&$row)
    {
        $parcnegocioService = InstanceProvider::getService()->getParcnegocio();
        $cdtipoinscricao = 'LIVRE';
        $nrinscricaoparc = null;

        if (!$row['NRPARCNEGOCIO']) {
            $data = $parcnegocioService->setParcnegocio($row['NMOPERADOR'], $nrinscricaoparc,
                $cdtipoinscricao, $row['CDOPERADOR']);
            $row['NRPARCNEGOCIO'] = $data['NRPARCNEGOCIO'];
        }
    }
}
