# Telegram

API do **Telegram** utilizando um **Bot** para enviar mensagens e arquivos.

Para mais informações, visite:
- https://core.telegram.org/bots/api
- https://github.com/TelegramBot/Api
- https://telegram-bot-sdk.readme.io/reference/getme

## Token

A comunicação com o **Bot** é feita utilizando um **token** de autenticação. Siga os passos abaixo para obtê-lo.

- Abra o aplicativo do **Telegram** e pesquise **BotFather**.

![BotFather](img/botfather.png)

> <span style="color: red;">*Nota:*</span>  *Utilize o que Bot verificado, como na imagem acima.*

- Envie o comando `/newbot` para criar um novo **Bot** e siga os passos que são solicitados, informando um **Nome** e um **Usuário**.

- Por fim, será emitida uma mensagem de sucesso contendo o **token** para autenticação, conforme imagem abaixo.

![Token](img/getToken.png)

> <span style="color: red;">**Atenção!**</span> O bot **NÃO INICIA UMA CONVERSA PRIVADA** e também **NÂO ENTRA EM UM GRUPO** através da API. Ele deve ser **ADICIONADO MANUALMENTE AO GRUPO** para que envie as mensagens. Para mensagens privadas, é necessário que o usuário **ENVIE UMA MENSAGEM PARA O BOT**. Somente dessa forma, o bot poderá enviar mensagens privada.

## Instalação via composer 

Adicione a biblioteca na sua aplicação com o comando abaixo:

```sh
composer require zeedhi/message-api
```

## Implementação

Foi implementada a classe [\Zeedhi\MessageApi\Telegram\Manager](src/Telegram/Manager.php) para facilitar o uso e disponibilizar funções úteis. Ela estende a classe 
[\TelegramBot\Api\BotApi](https://github.com/TelegramBot/Api/blob/master/src/BotApi.php), portanto, quaisquer funcionalidades estarão presentes em sua instância também.

##### Exemplo de uso:

```php
<?php
ini_set("memory_limit", "512M");
ini_set('max_execution_time', '260');
require_once __DIR__ . '/vendor/autoload.php';

header('Content-type: application/json; charset=utf-8');
$token = 'xxxxxxxxxx'; // Informe o token obtido nos passos anteriores
$bot   = new \Zeedhi\MessageApi\Telegram\Manager($token);

// Informações sobre o Bot
$botInfo   = $bot->getMe();
$botId     = $botInfo->getId();
$firstName = $botInfo->getFirstName();
$username  = $botInfo->getUsername();
var_dump($botId, $firstName, $username);
```

## Função extra

Na classe **Manager** foi implementada a função `getAllChatsByUpdates` que retorna um array associativo 
contendo todos os chats baseado nas suas 100 últimas atualizações, indexado pelo id do chat.
> <span style="color: red;">**Nota:**</span> A documentação da **API do Telegram** informa que as atualizações são das últimas 24 horas.

##### Exemplo de uso:

```php
<?php
ini_set("memory_limit", "512M");
ini_set('max_execution_time', '260');
require_once __DIR__ . '/vendor/autoload.php';

header('Content-type: application/json; charset=utf-8');
$token = 'xxxxxxxxxxxx';
$bot   = new \Zeedhi\MessageApi\Telegram\Manager($token);

$allChats = $bot->getAllChatsByUpdates();
foreach ($allChats as $chat) {
    // Define a mensagem de acordo com o tipo do chat
    if($chat->getType() == 'private') {
        $message = "Bom dia {$chat->getFirstName()} {$chat->getLastName()}";
    } else if($chat->getType() == 'group') {
        $message = "Bom dia a todos do meu grupo {$chat->getTitle()} querido";
    } else {
        $message = "Bom dia";
    }

    // Envia a mensagem
    $bot->sendMessage($chat->getId(), $message);
    // Envia uma imagem
    $bot->sendPhoto($chat->getId(), 'https://encrypted-tbn0.gstatic.com/images?q=tbn:ANd9GcQbJLKNkDdPaVjw8-tL2hVuQJJzn0GFvrABAQ&usqp=CAU');
}
```

## Enviar mensagem para um grupo

- Crie um grupo pelo aplicativo do **Telegram**.

![img.png](img/novoGrupo.png)

- Adicione e bot pesquisando pelo **Usuário** informado durante sua criação nos passos anteriores.
- Adicione também todas as pessoas que deverãos receber as mensagens enviadas pelo bot.

![img.png](img/addBot.png)

- Utilize a função `getAllChatsByUpdates` após adicionar o bot 
no grupo para obter o id do chat desse grupo, que será utilizado pelo bot para enviar as mensagens.

> <span style="color: red;">**Nota:**</span> Atentar-se para o prazo de 24h de validade das mensagens.

##### Exemplo:

```php
<?php
ini_set("memory_limit", "512M");
ini_set('max_execution_time', '260');
require_once __DIR__ . '/vendor/autoload.php';

header('Content-type: application/json; charset=utf-8');
$token = 'xxxxxxxxxxxx';
$bot   = new \Zeedhi\MessageApi\Telegram\Manager($token);

$allChats = $bot->getAllChatsByUpdates();
foreach ($allChats as $chat) {
    if ($chat->getType() == 'group') {
        if ($chat->getTitle() === 'Grupo do bot') {
            // crie a função saveGroup para registrar o id do chat no banco de dados
            saveGroup($chat->getId(), $chat->getTitle());
        }
    }
}
```

- Em posse do **ID do chat** e do **Token**, será possível enviar as mensagens.

##### Exemplo:

```php
<?php
ini_set("memory_limit", "512M");
ini_set('max_execution_time', '260');
require_once __DIR__ . '/vendor/autoload.php';

header('Content-type: application/json; charset=utf-8');
$token = 'xxxxxxxxxxxx';
$bot   = new \Zeedhi\MessageApi\Telegram\Manager($token);

$title   = 'Grupo do bot';
$message = 'Bom dia';
 // crie a função getIdChatGroup para buscar o id do chat no banco de dados
$idChat = getIdChatGroup($title);
$bot->sendMessage($idChat, $message);
```