import { Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Modal, ModalService, IGridEvent, TextInput, SelectMultiple, Select, Textarea, Form } from '@zeedhi/common';
import { PmLibAlertService, PmLibFormSave, PmLibHttp, PmLibDialogService } from '@zeedhi/pm-lib';
import AclMessages from '@/Util/AclMessages';
import { OperadorService } from '@/services/OperadorService';

export class ConfigBotControllerGrid {
    public filterDate: any;

    private modal!: Modal;

    public promiseConfigBotInfo: any = null;

    public addConfigBotModalPath: string = 'components/ConfiguracoesDeBot/grid/modais/addCon';

    public operadorService: OperadorService;

	public currentRow: any = {};

    constructor() {
        this.operadorService = new OperadorService();
	}

	public getInstanceConfigBotGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('configuracoesDeBot');
	}

	public deleteGridConfigBot(): TekGrid {
		return Metadata.getInstance<TekGrid>('configuracoesDeBot');
	}

    public async openConfigBotGrid({ row }: any) {
		this.currentRow = row;
        this.modal = await ModalService.createFromJson('components/configuracoes-de-bot/modais/gridmodal', true);
        this.modal.show();
    }

    public async addConfigBotGrid() {
        this.modal = await ModalService.createFromJson('components/configuracoes-de-bot/modais/addBot', true);
        this.modal.show();
    }

    public closeConfigBotGrid() {
        this.modal.hide();
        this.modal?.destroy();
    }

    public async saveaddConfigBotGrid() {
		const addForm = Metadata.getInstance<PmLibFormSave>('addConfigBotModal');
		const addConfigBotGrid = this.getInstanceConfigBotGrid();
		let tokenValidate = true;
		addConfigBotGrid.datasource.data.forEach((row: any) => {
			if (row.TOKEN === Metadata.getInstance<TextInput>('TOKEN').value) {
				tokenValidate = false;
			}
		});
		if (addForm.validate()) {
			if (tokenValidate) {
				await PmLibHttp.$post('/saveConfigBot', {
					TOKEN: Metadata.getInstance<TextInput>('TOKEN').value,
					BOT_ID: Metadata.getInstance<TextInput>('BOT_ID').value,
					IDATIVO: Metadata.getInstance<Select>('IDATIVO').value,
					NAME: Metadata.getInstance<TextInput>('NAME').value,
					USERNAME: Metadata.getInstance<TextInput>('USERNAME').value,
					GROUPS: Metadata.getInstance<SelectMultiple>('GROUPS').value,
					DESCRIPTION: Metadata.getInstance<Textarea>('DESCRIPTION').value,
				});
				this.closeConfigBotGrid();
				addConfigBotGrid.reload();
				PmLibAlertService.showSuccess(AclMessages.SUCCESS_SAVE_DEAFULT);
			} else {
				PmLibAlertService.showError(AclMessages.ERROR_TOKEN);
			}
		}
	}

	public async saveModalConfigBotGrid() {
		const addForm = Metadata.getInstance<PmLibFormSave>('configBotModal');
		const addConfigBotGrid = this.getInstanceConfigBotGrid();
		if (addForm.validate()) {
			await PmLibHttp.$post('/saveConfigBot', {
				ID: this.currentRow.ID,
				GROUP_ID: this.currentRow.GROUP_ID,
				TOKEN: Metadata.getInstance<TextInput>('TOKEN').value,
				BOT_ID: Metadata.getInstance<TextInput>('BOT_ID').value,
				IDATIVO: Metadata.getInstance<Select>('IDATIVO').value,
				NAME: Metadata.getInstance<TextInput>('NAME').value,
				USERNAME: Metadata.getInstance<TextInput>('USERNAME').value,
				GROUPS: Metadata.getInstance<SelectMultiple>('GROUPS').value,
				DESCRIPTION: Metadata.getInstance<Textarea>('DESCRIPTION').value,
			});
			this.closeConfigBotGrid();
			addConfigBotGrid.reload();
			PmLibAlertService.showSuccess(AclMessages.SUCCESS_SAVE_DEAFULT);
		}
	}

	public async beforeDeleteConfigBotGrid({ event }: IGridEvent) {
		event?.preventDefault();
		const confirm = await PmLibDialogService.confirmDialog(AclMessages.DELETE_DEFAULT);
		if (confirm) {
			this.deleteConfigBotGrid();
		}
	}

	private async deleteConfigBotGrid() {
		const grid = this.getInstanceConfigBotGrid();
		await PmLibHttp.$delete('/deleteConfigBot', { data: grid.selectedRows });
		PmLibAlertService.showSuccess(AclMessages.SUCCESS_DELETE_DEAFULT);
		await grid.reload();
	}

	public async setCamposModalAdd({ component }: any) {
		let botId = null;
		let name = null;
		let username = null;
		const groups = Metadata.getInstance<SelectMultiple>('GROUPS');
		groups.datasource.data = [];
		if (component.value !== null) {
			const response = await PmLibHttp.$post('/getDateBot', {
				TOKEN: component.value,
			});
			botId = response.data.BOT_ID;
			name = response.data.NAME;
			username = response.data.USERNAME;
            groups.datasource.setFilter({
                TOKEN: component.value,
            });
		}
		Metadata.getInstance<TextInput>('BOT_ID').value = botId;
		Metadata.getInstance<TextInput>('NAME').value = name;
		Metadata.getInstance<TextInput>('USERNAME').value = username;
	}

	public async setCamposGridModal() {
		const groups = Metadata.getInstance<SelectMultiple>('GROUPS');
		await groups.datasource.setFilter({
			TOKEN: this.currentRow.TOKEN,
		});
		const idGroup = this.currentRow.ID_GROUP.map(Number);
		Metadata.getInstance<TextInput>('TOKEN').value = this.currentRow.TOKEN;
		Metadata.getInstance<TextInput>('BOT_ID').value = this.currentRow.BOT_ID;
		Metadata.getInstance<TextInput>('NAME').value = this.currentRow.NAME;
		Metadata.getInstance<TextInput>('USERNAME').value = this.currentRow.USERNAME;
		Metadata.getInstance<Select>('IDATIVO').value = this.currentRow.IDATIVO;
		Metadata.getInstance<Textarea>('DESCRIPTION').value = this.currentRow.DESCRIPTION;
		Metadata.getInstance<SelectMultiple>('GROUPS').setValue(idGroup);
	}

	public async setCamposAuditoria() {
		Metadata.getInstance<TextInput>('DTINCLUSAO').value = this.currentRow.DTINCLUSAO;
		Metadata.getInstance<TextInput>('NMOPERINCLUSAO').value = this.currentRow.NMOPERINCLUSAO;
		Metadata.getInstance<TextInput>('DTULTATU').value = this.currentRow.DTULTATU;
		Metadata.getInstance<TextInput>('NMOPERULTATU').value = this.currentRow.NMOPERULTATU;
	}

	public clickButtonToExpand({ component }: any) {
        const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
        form.isVisible = !form.isVisible;
        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }
}
