import { Loader, Metadata, Singleton } from '@zeedhi/core';
import { OperadorController } from '@/controllers/Operador/OperadorController';
import { OperadorService } from '@/services/OperadorService';
import { PmLibAlertService, PmLibFormSave, PmLibHttp, PmLibUtil } from '@zeedhi/pm-lib';
import AclMessages from '@/Util/AclMessages';
import { Form } from '@zeedhi/common';

@Singleton
export class TabEditOperadorController {
	public operadorControler: OperadorController;

	public operadorService: OperadorService;

	get visibleOnlyInCompleteMode(): boolean {
		return this.operadorControler.showFullOperatorWindow;
	}

	constructor() {
		this.operadorControler = Loader.getInstance('OperadorController');
		this.operadorService = new OperadorService();
	}

	public getOperadorFormEdit(): PmLibFormSave {
		return Metadata.getInstance<PmLibFormSave>('operadorFormEdit');
	}

	public async saveEditOperador() {
		const form = this.getOperadorFormEdit();
		if (form.validate()) {
			form.value.NRINSCRICAOPARC = PmLibUtil.getOnlyNumberInString(form.value.NRINSCRICAOPARC);
			form.value.NRPERFILACESSO = localStorage.getItem('CDPERFILSELECIONADO');
			form.value.CDTIPOINSCRICAO = this.operadorService.getInscriptionType(form.value.NRINSCRICAOPARC);
			form.value.SHOW_FULL_OPERATOR = (window.showFullOperatorWindow || false);
			form.value.IDOPERATIVO = (form.value.IDATIVO === 'S') ? 'A' : 'I';
			form.value.ADMINISTRATOR = false;

			form.value.OLD_CPF = this.operadorControler.backupRow.NRINSCRICAOPARC;
			form.value.OLD_CDCLIENTE = this.operadorControler.backupRow.CDCLIENTE;
			form.value.OLD_NMOPERADOR = this.operadorControler.backupRow.NMOPERADOR;
			this.operadorControler.isReloadOperadorGrid = true;
			await PmLibHttp.$put('/operador', form.value);
			PmLibAlertService.showSuccess(AclMessages.SUCCESS_SAVE_DEAFULT);
		}
	}

	public clickButtonToExpand({ component }: any) {
        const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
        form.isVisible = !form.isVisible;
        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }
}
